@@ -17,7 +17,6 @@
  */
 #include "sysincludes.h"
 #include "mtools.h"
-#include "codepage.h"
 #include "mtoolsPaths.h"
 
 /* global variables */
@@ -49,7 +48,7 @@ static int cur_dev; /* device being filled in. If negative, none */
 static int trusted=0; /* is the currently parsed device entry trusted? */
 static unsigned int nr_dev; /* number of devices that the current table can
 			       hold */
-struct device *devices; /* the device table */
+struct device *devices=NULL; /* the device table */
 static int token_nr; /* number of tokens in line */
 
 static char default_drive='\0'; /* default drive */
@@ -67,7 +66,6 @@ unsigned int mtools_twenty_four_hour_clock=1;
 unsigned int mtools_lock_timeout=30;
 unsigned int mtools_default_codepage=850;
 const char *mtools_date_string="yyyy-mm-dd";
-char *country_string=0;
 
 typedef struct switches_l {
     const char *name;
@@ -78,7 +76,8 @@ typedef struct switches_l {
 	T_UINT,
 	T_UINT8,
 	T_UINT16,
-	T_UQSTRING
+	T_UQSTRING,
+	T_OFFS
     } type;
 } switches_t;
 
@@ -164,7 +163,7 @@ static struct {
 
 static switches_t dswitches[]= {
     { "FILE", OFFS(name), T_STRING },
-    { "OFFSET", OFFS(offset), T_UINT },
+    { "OFFSET", OFFS(offset), T_OFFS },
     { "PARTITION", OFFS(partition), T_UINT },
     { "FAT", OFFS(fat_bits), T_INT },
     { "FAT_BITS", OFFS(fat_bits), T_UINT },
@@ -175,6 +174,7 @@ static switches_t dswitches[]= {
     { "SECTORS", OFFS(sectors), T_UINT16 },
     { "HIDDEN", OFFS(hidden), T_UINT },
     { "PRECMD", OFFS(precmd), T_STRING },
+    { "POSTCMD", OFFS(postcmd), T_STRING },
     { "BLOCKSIZE", OFFS(blocksize), T_UINT },
     { "CODEPAGE", OFFS(codepage), T_UINT },
     { "DATA_MAP", OFFS(data_map), T_UQSTRING }
@@ -253,8 +253,10 @@ static void syntax(const char *msg, int thisLine)
     exit(1);
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-align"
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wcast-align"
+#endif
 static void get_env_conf(void)
 {
     char *s;
@@ -266,7 +268,7 @@ static void get_env_conf(void)
 	    errno = 0;
 	    switch(global_switches[i].type) {
 	    case T_INT:
-		* ((int *)global_switches[i].address) = strtoi(s,0,0);
+		* ((int *)global_switches[i].address) = strtosi(s,0,0);
 		break;
 	    case T_UINT:
 		* ((unsigned int *)global_switches[i].address) = strtoui(s,0,0);
@@ -281,6 +283,9 @@ static void get_env_conf(void)
 	    case T_UQSTRING:
 		* ((char **)global_switches[i].address) = s;
 		break;
+	    case T_OFFS:
+		* ((mt_off_t *)global_switches[i].address) = str_to_offset(s);
+		break;
 	    }
 	    if(errno != 0) {
 		fprintf(stderr, "Bad number %s for %s (%s)\n", s,
@@ -291,7 +296,9 @@ static void get_env_conf(void)
 	}
     }
 }
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
 #pragma GCC diagnostic pop
+#endif
 
 static int mtools_getline(void)
 {
@@ -418,6 +425,23 @@ static int get_number(void)
     return n;
 }
 
+static mt_off_t get_offset(void)
+{
+    char *last;
+    mt_off_t n;
+
+    skip_junk(1);
+    last = pos;
+    n= str_to_off_with_end(pos, &pos);
+    if(errno)
+	syntax("bad number", 0);
+    if(last == pos)
+	syntax("numeral expected", 0);
+    pos++;
+    token_nr++;
+    return n;
+}
+
 /* purge all entries pertaining to a given drive from the table */
 static void purge(char drive, int fn)
 {
@@ -505,8 +529,11 @@ static void finish_drive_clause(void)
     cur_dev = -1;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-align"
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wcast-align"
+#endif
+
 static int set_var(struct switches_l *switches, int nr,
 		   caddr_t base_address)
 {
@@ -537,12 +564,17 @@ static int set_var(struct switches_l *switches, int nr,
 	    else if (switches[i].type == T_UQSTRING)
 		* ((char**)((long)switches[i].address+base_address))=
 		    get_unquoted_string();
+	    else if (switches[i].type == T_OFFS)
+		* ((mt_off_t*)((long)switches[i].address+base_address))=
+		    get_offset();
 	    return 0;
 	}
     }
     return 1;
 }
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
 #pragma GCC diagnostic pop
+#endif
 
 static int set_openflags(struct device *dev)
 {
@@ -670,7 +702,7 @@ static void parse_old_device_line(char drive)
 {
     char name[MAXPATHLEN];
     int items;
-    long offset;
+    mt_off_t offset;
 
     int heads, sectors, tracks;
 
@@ -682,13 +714,13 @@ static void parse_old_device_line(char drive)
 
     /* reserve slot */
     append();
-    items = sscanf(token,"%c %s %i %i %i %i %li",
+    items = sscanf(token,"%c %s %i %i %i %i " IMTOF,
 		   &devices[cur_dev].drive,name,&devices[cur_dev].fat_bits,
 		   &tracks,&heads,&sectors, &offset);
     devices[cur_dev].heads = tou16(heads, "heads");
     devices[cur_dev].sectors = tou16(sectors, "sectors");
     devices[cur_dev].tracks = (unsigned int) tracks;
-    devices[cur_dev].offset = (off_t) offset;
+    devices[cur_dev].offset = offset;
     switch(items){
 	case 2:
 	    devices[cur_dev].fat_bits = 0;
@@ -888,7 +920,7 @@ void mtoolstest(int argc, char **argv, int type  UNUSEDP)
 	       dev->name,dev->fat_bits);
 	printf("\ttracks=%d heads=%d sectors=%d hidden=%d\n",
 	       dev->tracks, dev->heads, dev->sectors, dev->hidden);
-	printf("\toffset=0x%lx\n", (long) dev->offset);
+	printf("\toffset=" XMTOF "\n", dev->offset);
 	printf("\tpartition=%d\n", dev->partition);
 
 	if(dev->misc_flags)
